/************************************************************************************************************************* 
 *                                                                                                                       *
 *      _____        __ _______                                                                                          *
 *     / ____|      / _|__   __|                       by Tom Schilli                                                    *
 *    | (___   ___ | |_   | |_      ____ _ _ __ ___                                                                      *
 *     \___ \ / _ \|  _|  | \ \ /\ / / _` | '__/ _ \   Public Domain Software: Use as you like.                          *
 *     ____) | (_) | |_   | |\ V  V / (_| | | |  __/                                                                     *
 *    |_____/ \___/|_(_)  |_| \_/\_/ \__,_|_|  \___|   http://sof.tware.de                                               *
 *                                                                                                                       *
 *                                                                                                                       *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,                                   *
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,               *
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE          *
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                                                         *
 *                                                                                                                       *
 *************************************************************************************************************************/



#define DEBUG_ARRAY_SIZE 20

unsigned int debugData_delta[DEBUG_ARRAY_SIZE];
unsigned int debugData_counter[DEBUG_ARRAY_SIZE];

unsigned int pos;

void setup() {
  // Init serial console
  Serial.begin(9600);

  // init debug variables
  pos=0;
  memset(debugData_delta,0,sizeof(debugData_delta));
  memset(debugData_counter,0,sizeof(debugData_counter));

  // Enable Input Capture
  TIMSK1 = ( (1<<ICIE1) );                          // Enable "Input Capture Pin" Interrupt
  
  TCCR1A = 0;                                       // clear register to have all 16bit counter (TOP=0xFFFF).

  TCCR1B = ( (1<<ICES1) | (1<<ICNC1) | (1<<CS12));  // Prescaler 256 (-> 16MHz/256 = 62.5kHz, means 16µs per Tick.
                                                    // Trigger on rising edge
                                                    // enable noise canceler
}


/**
 * Print Debug Messages on UART
 */
void printDebugMessages() {

  char buffer[25];

  // Write the Debug Array content in a nice way
  Serial.write("-----------\n");
  (void)sprintf(buffer,"Pos=%3u | Counter | Delta\n",pos);
  Serial.print(buffer);
  
  for (int i = 0; i<DEBUG_ARRAY_SIZE; i++) {
    (void)sprintf(buffer, "I= %3u  :  %5u  : %5u\n", i, debugData_counter[i], debugData_delta[i]);
     Serial.print(buffer);
  }
  
}




/**
 * Main Loop: simply print debug message every second.
 */
void loop() {
  // put your main code here, to run repeatedly:
  printDebugMessages();
  delay(1000);

}



/**
 * INTERRUPT ROUTINE
 * Input Capture Pin
 */
ISR(TIMER1_CAPT_vect)
{
 // Counter variable for last tick must be static
 // because we want to keep the old value for the
 // next function call.
 uint16_t ticks_t1=0;
 static uint16_t lastTick=0;

  // Read the 16bit counter
  ticks_t1 = ICR1;

  // calculate time difference. 
  // Don't care about the overflow: 
  // Because it's modulo 0xFFFF logic it works automatically.
  unsigned int delta = ticks_t1 - lastTick;

  // Store to Debug Array for console output.
  debugData_counter[pos] = ticks_t1;
  debugData_delta[pos] = delta;
  // shift debug array position pointer to next location
  pos++;
  if (pos>=DEBUG_ARRAY_SIZE) pos=0;     

  // store last timer for next call.
  lastTick = ticks_t1;
}
